ORG %DDDDDD 					;  Beginning of memory
@irq		DT $0
@.data		DT $0

ORG %DDDDDB
@redraw 	DT $0				; Screen repaints when
						; different from 0
; -------------------------------------------------------------
ORG %DDCDDD					; Stack page
; --------------------------------------------------------------
ORG %DDBDDD					; Screen buffer
@screen 	
@.page1
		INC 	'ram/screen.asm'	; Initial screen configuration
ORG %DDA000
@.page2

ORG %DD0DDD
@.end						; End of screen	

; ---------------------------------------------------------------

ORG %000000					; Initial PC value
@null		
		NOP
@inloop
		LDA 	@counter
		JSR 	@putnon

		LDA 	@counter+$1
		JSR 	@putnon

		LAD 	@prompt			; Print "> "
		JSR 	@puts

		JSR 	@getstring		; Wait for input
		LDX 	#$0

		LDA	@counter
		INC	@counter+$1
		ADD	#$0
		STA	@counter
@.loop		
		PHX				; Save X
		LAD 	(@commands, X)		; Load command string

		TYA
		JEQ 	@.invalid		; Break loop if null

		PHX				; Push command string
		PHY
		LAD 	@getstring.buffer	; Push entered text
		PHX
		PHY
		JSR 	@strcmp			; Compare

		PLX				; Restore X
		LDA 	A			; Load A into A (refresh flags)

		JNE 	+$6			; If not a match, continue
		JSR 	(@commands + $2 ,X) 	; Run the function
		JMP 	@inloop			; 

						; Increase X by 4
		TXA
		CLC
		ADD 	#$4
		TAX
						; Keep comparing
		JMP 	@.loop
		
@.invalid
		JSR 	@unknown
		JMP 	@inloop

@commands	DW 	@help.txt , @help, @charmap.txt , @charmap
		DW 	$0, $0



@uppercase	DT 	'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
		DT 	$0

@help
		LAD 	@.message
		JSR 	@puts
		JSR 	@feedscreen
		RST

@.message 	DT 	'Available commands=HELP,CHARMAP', $0
@.txt		DT 	'HELP', $0

@charmap
		LAD	@.info			; Print info message
		JSR	@puts
		JSR	@feedscreen
		JSR	@feedscreen

		LDA	#$0			; Indent by 1
		JSR	@putchar

		LAD	@.dec			; Print 0123456789
		JSR	@puts
		LDA	#$0

@.loop		PSH	A
		TAX
		MOD	#$10			
		JNE	@.sc
						; Only run if A is 10,20,30,...
		JSR 	@feedscreen
		PLL	A
		PSH	A
		DIV	#$10
		TAX				; Print left-column index
		LDA	@.dec,X
		JSR	@putchar
@.sc
		PLL	A
		JSR	@putchar		; Print character
		INC	A			; Increase A
		CMP	#$100			; Loop till A = 100
		JLT	@.loop

		JSR	@feedscreen		; Scroll a bit
		JSR	@feedscreen

		RST
		
@.dec		DT	'0123456789', $0
@.txt		DT	'CHARMAP', $0
@.info		DT	'Character value = vertical*10+horizontal', $0

@unknown
		LAD 	@.message
		JSR 	@puts
		JSR 	@feedscreen
		RST
@.message 	DT 	'Unknown command', $0

@counter	DW $0

; -- include standard functions
INC	'ram/stdlib.asm'
INC	'ram/math.asm'
; --

ORG %444DDD
; Interrupt handler
; (This needs to be fairly breief, since it only gets just under one page)
;
@interrupthandler
		LDA 	@irq			; Load IRQ into A
		CMP 	#$0			; IRQ = 0 => Keyboard
		JEQ 	@keyboard
		CMP 	#$1			; IRQ = 1 => Clock
		JEQ 	@clock

		; Unknown interrupt
		LAD 	@unkmsg
		JSR 	@puts 
		JSR 	@feedscreen
		RTI

@clock		INC 	@redraw			; Tell the screen to redraw
		RTI
; Keyboard interrupt
@keyboard	LDA 	@irq.data
		CMP 	#$1			; Space
		JEQ 	@.disp
		CMP 	#$9			; > 9 => Regular symbol
		JGT 	@.disp

		CMP 	#$2			; New line
		JEQ 	@.newline

@.huh		RTI

@.disp		LDA 	@irq.data		; Print to screen
		STA 	@printbuf
		LAD 	@printbuf
		JSR 	@puts

		LDA 	#$0			; Disable "got a new line"-flag
		STA 	@kb_got_line

		LDA 	@kbuf_length		; Buffer only holds 39 chars
		CMP 	#$39
		JGT 	@.huh
		TAX

		LDA 	@irq.data		; Load input to A
		STA 	@keybuffer,X		; Put it in buffer
		INX				; Increase buffer length
		LDA 	#$0
		STA 	@keybuffer,X		; Zero-terminate string
		STX 	@kbuf_length		; Save buffer length

		RTI
@.newline
		JSR 	@feedscreen		; Scroll the screen down

		LDX 	@kbuf_length
		LDA 	#$0
		STA 	@keybuffer,X
		STA 	@kbuf_length		; Reset buf length to zero

		LDA 	#$1			; Enable "got a new line" flag
		STA 	@kb_got_line

		RTI
		

@prompt		DT 	'> ', $0
@printbuf	DT 	$0 $0
@unkmsg		DT 	'UNKNOWN INTERRUPT' , $0

; ----------------------------------------------------------------

ORG %444441
		DT 	%444			; Set clock to once every
						; 364 instructions
ORG %444442
@irvector 	DW 	@interrupthandler	; Interrupt handler vector
ORG %444444
@endcode					; End of memory
