/* Tunguska, ternary virtual machine
 *
 * Copyright (C) 2007,2008 Viktor Lofgren
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SHARE_DIR
# define SHARE_DIR "/usr/local/share/tunguska/"
#endif
#include "display.h"

display::display() {
	SDL_Init(SDL_INIT_VIDEO);
	SDL_WM_SetCaption("TernCPU", 0);
	window = SDL_SetVideoMode(XRES, YRES, 16, SDL_HWSURFACE);

	symbols = SDL_LoadBMP(SHARE_DIR "symbols.bmp");

	if(symbols == NULL) {
		printf("'symbols.bmp' not found in '%s'\n", SHARE_DIR);
		printf("Falling back to 'share/symbols.bmp' ...");
		symbols = SDL_LoadBMP("share/symbols.bmp");
		if(symbols == NULL) {
			printf(" failure :-(\n");
			exit(EXIT_FAILURE);
		} else printf(" success!\n");
	}
			
	memset(buffer, 101, sizeof(buffer));
	SDL_Rect r = { 0, 0, XRES, YRES };
	SDL_FillRect(window, &r, SDL_MapRGB(window->format, 36, 36, 36));
	SDL_Flip(window);

	atexit(SDL_Quit);
}

display::~display() {
	SDL_Quit();

}

void display::putcxy(int character, int x, int y) {
	int c = (y * COLS + x) % (ROWS*COLS);
	if(buffer[c] == character) return;
	buffer[c] = character;
	if(character < 0 || character > 100) return;
	
	SDL_Rect src = { 8*(character % 10), (character / 10) * 10, 8, 10 };
       	SDL_Rect dst = { XRES / 2 - COLS * 4 + 8 * x, YRES / 2 - ROWS * 5 + 10 * y, 8, 10 };
       	SDL_BlitSurface(symbols, &src, window, &dst);

	/* Updating just a small segment of the screen is
	 * way, WAY much faster than flipping the entire screen.
	 */
	SDL_UpdateRect(window, dst.x, dst.y, dst.w, dst.h);
}

void display::printscreen(machine& m) {
	for(int x = 0; x < COLS; x++) {
		for(int y = 0; y < ROWS; y++) {
			putcxy(m.memref(-264262 + x + COLS*y).to_int(), x, y);
		}
	}
}
