/* Tunguska, ternary virtual machine
 *
 * Copyright (C) 2007,2008 Viktor Lofgren
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include "machine.h"
#include "keyboard.h"

/* Poll events every POLL_INTERAL instructions. 
 *
 * 5000 is a good compromise between a short interval in which
 * most of the time is spent polling I/O, and a long interval,
 * in which there is a pronounced delay between keypress and response.
 */
#define POLL_INTERVAL 5000

void usage(char* app) {
	printf("Usage: %s [-h] [-f] [image]\n"
		"\t-f Fullscreen\n"
		"\t-h Show this message\n\n", app);
}
int main(int argc, char* argv[]) {
	SDL_Init(SDL_INIT_VIDEO); // Initialize video (and keyboard)

	int fullscreen = 0;
	int optret;
	while((optret = getopt(argc, argv, "hf")) != -1) {
		switch(optret) {
			case 'h': usage(argv[0]); exit(EXIT_SUCCESS); 
			case 'f': fullscreen = 1; break;
			case '?': usage(argv[1]); exit(EXIT_FAILURE);
			case ':': usage(argv[1]); exit(EXIT_FAILURE);
		}
	}


	machine *m = new machine();
	kboard *keyboard = new kboard(m);
	display *disp = new display();

	if(fullscreen) disp->fullscreen();

	if(argc > optind) {
		printf("LOADING %s\n", argv[optind]);
		m->load(argv[optind]);
		printf("...DONE\n");
	} else {
		printf("Tunguska loaded null memory, basically\n"
		       "rendering it a drooling vegetable :-(\n"
		       "\n"
		       "It's a lot more fun with something in the\n"
		       "memory to execute...");
	}

	srandom(time(NULL));

	/* Initialization is done */

	int ioclock = 0;
	for(;;) {
		if(ioclock++ > POLL_INTERVAL) {
		       	keyboard->poll();
			ioclock = 0;
		}

		if(m->P[machine::B].to_int() == 0 && m->P[machine::I].to_int() == 0) {
			if(keyboard->has_input()) {
				m->memref("DDD", "DDD") = machine::IRQ_KEYBOARD;
				m->memref("DDD", "DDC") = keyboard->get_input();
				m->brk();
			} else if(keyboard->has_break()) {
				m->memref("DDD", "DDD") = machine::IRQ_KEYBREAK;
				m->memref("DDD", "DDC") = 0;
				m->brk();
			} else if(m->CL.to_int() == 0) {
				m->memref("DDD", "DDD") = machine::IRQ_CLOCK;
				m->memref("DDD", "DDC") = random();
				m->brk();
			}
		}
		if(m->memref("DDD", "DDB").to_int() != 0) {
			m->memref("DDD", "DDB") = 0;
			disp->printscreen(*m);
		}

		if(m->autorun) {
			m->instruction();
		}
	}
}
